@@ -27,16 +27,13 @@
  *******************************************************************************************************/
 
 #include "DkManipulators.h"
-#include "DkManipulatorsIpl.h"
 
-#include "DkImageContainer.h"
 #include "DkImageStorage.h"
+#include "DkManipulatorsIpl.h"
 #include "DkSettings.h"
 
-#pragma warning(push, 0) // no warnings from includes
 #include <QSharedPointer>
 #include <QWidget>
-#pragma warning(pop)
 
 namespace nmc
 {
@@ -48,8 +45,7 @@ DkBaseManipulator::DkBaseManipulator(QAction *action)
 
     // add default icon
     if (mAction->icon().isNull()) {
-        QSize size(22, 22);
-        mAction->setIcon(DkImage::loadIcon(":/nomacs/img/sliders.svg", size));
+        mAction->setIcon(DkImage::loadIcon(":/nomacs/img/sliders.svg"));
     }
 }
 
@@ -65,9 +61,7 @@ QAction *DkBaseManipulator::action() const
 }
 
 // DkManipulatorManager --------------------------------------------------------------------
-DkManipulatorManager::DkManipulatorManager()
-{
-}
+DkManipulatorManager::DkManipulatorManager() = default;
 
 void DkManipulatorManager::createManipulators(QWidget *parent)
 {
@@ -77,84 +71,84 @@ void DkManipulatorManager::createManipulators(QWidget *parent)
     QVector<QSharedPointer<DkBaseManipulator>> mpls;
     mpls.resize(m_ext_end);
 
-    QSize size(22, 22);
-
     // grayscale
     QAction *action;
-    action = new QAction(DkImage::loadIcon(":/nomacs/img/grayscale.svg", size), QObject::tr("&Grayscale"), parent);
+    action = new QAction(DkImage::loadIcon(":/nomacs/img/grayscale.svg"), QObject::tr("&Grayscale"), parent);
     action->setStatusTip(QObject::tr("Convert to Grayscale"));
     mpls[m_grayscale] = QSharedPointer<DkGrayScaleManipulator>::create(action);
 
     // auto adjust
-    action = new QAction(DkImage::loadIcon(":/nomacs/img/auto-adjust.svg", size), QObject::tr("&Auto Adjust"), parent);
+    action = new QAction(DkImage::loadIcon(":/nomacs/img/auto-adjust.svg"), QObject::tr("&Auto Adjust"), parent);
     action->setShortcut(Qt::CTRL | Qt::SHIFT | Qt::Key_L);
     action->setStatusTip(QObject::tr("Auto Adjust Image Contrast and Color Balance"));
     mpls[m_auto_adjust] = QSharedPointer<DkAutoAdjustManipulator>::create(action);
 
     // normalize
-    action = new QAction(DkImage::loadIcon(":/nomacs/img/normalize.svg", size), QObject::tr("Nor&malize Image"), parent);
+    action = new QAction(DkImage::loadIcon(":/nomacs/img/normalize.svg"), QObject::tr("Nor&malize Image"), parent);
     action->setShortcut(Qt::CTRL | Qt::SHIFT | Qt::Key_N);
     action->setStatusTip(QObject::tr("Normalize the Image"));
     mpls[m_normalize] = QSharedPointer<DkNormalizeManipulator>::create(action);
 
     // flip horizontal
-    action = new QAction(DkImage::loadIcon(":/nomacs/img/flip-horizontal.svg", size), QObject::tr("Flip &Horizontal"), parent);
+    action = new QAction(DkImage::loadIcon(":/nomacs/img/flip-horizontal.svg"),
+                         QObject::tr("Flip &Horizontal"),
+                         parent);
     action->setStatusTip(QObject::tr("Flip Image Horizontally"));
     mpls[m_flip_h] = QSharedPointer<DkFlipHManipulator>::create(action);
 
     // flip vertical
-    action = new QAction(DkImage::loadIcon(":/nomacs/img/flip-vertical.svg", size), QObject::tr("Flip &Vertical"), parent);
+    action = new QAction(DkImage::loadIcon(":/nomacs/img/flip-vertical.svg"), QObject::tr("Flip &Vertical"), parent);
     action->setStatusTip(QObject::tr("Flip Image Vertically"));
     mpls[m_flip_v] = QSharedPointer<DkFlipVManipulator>::create(action);
 
     // invert image
-    action = new QAction(DkImage::loadIcon(":/nomacs/img/invert.svg", size), QObject::tr("&Invert Image"), parent);
+    action = new QAction(DkImage::loadIcon(":/nomacs/img/invert.svg"), QObject::tr("&Invert Image"), parent);
     action->setStatusTip(QObject::tr("Invert the Image"));
     mpls[m_invert] = QSharedPointer<DkInvertManipulator>::create(action);
 
     // extended --------------------------------------------------------------------
     // tiny planet
-    action = new QAction(DkImage::loadIcon(":/nomacs/img/tiny-planet.svg", size), QObject::tr("&Tiny Planet..."), parent);
+    action = new QAction(DkImage::loadIcon(":/nomacs/img/tiny-planet.svg"), QObject::tr("&Tiny Planet..."), parent);
     action->setStatusTip(QObject::tr("Create a Tiny Planet"));
     mpls[m_tiny_planet] = QSharedPointer<DkTinyPlanetManipulator>::create(action);
 
     // tiny planet
-    action = new QAction(DkImage::loadIcon(":/nomacs/img/bucket.svg", size), QObject::tr("&Background Color..."), parent);
+    action = new QAction(DkImage::loadIcon(":/nomacs/img/bucket.svg"), QObject::tr("&Background Color..."), parent);
     action->setStatusTip(QObject::tr("Add a background color"));
     mpls[m_color] = QSharedPointer<DkColorManipulator>::create(action);
 
     // blur
-    action = new QAction(DkImage::loadIcon(":/nomacs/img/blur.svg", size), QObject::tr("&Blur..."), parent);
+    action = new QAction(DkImage::loadIcon(":/nomacs/img/blur.svg"), QObject::tr("&Blur..."), parent);
     action->setStatusTip(QObject::tr("Blur the image"));
     mpls[m_blur] = QSharedPointer<DkBlurManipulator>::create(action);
 
     // unsharp mask
-    action = new QAction(DkImage::loadIcon(":/nomacs/img/sharpen.svg", size), QObject::tr("&Sharpen..."), parent);
+    action = new QAction(DkImage::loadIcon(":/nomacs/img/sharpen.svg"), QObject::tr("&Sharpen..."), parent);
     action->setStatusTip(QObject::tr("Sharpens the image by applying an unsharp mask"));
     mpls[m_unsharp_mask] = QSharedPointer<DkUnsharpMaskManipulator>::create(action);
 
     // rotate
-    action = new QAction(DkImage::loadIcon(":/nomacs/img/rotate-cc.svg", size), QObject::tr("&Rotate..."), parent);
+    action = new QAction(DkImage::loadIcon(":/nomacs/img/rotate-cc.svg"), QObject::tr("&Rotate..."), parent);
     action->setStatusTip(QObject::tr("Rotate the image"));
     mpls[m_rotate] = QSharedPointer<DkRotateManipulator>::create(action);
 
     // resize
-    action = new QAction(DkImage::loadIcon(":/nomacs/img/resize.svg", size), QObject::tr("&Resize..."), parent);
+    action = new QAction(DkImage::loadIcon(":/nomacs/img/resize.svg"), QObject::tr("&Resize..."), parent);
     action->setStatusTip(QObject::tr("Resize the image"));
     mpls[m_resize] = QSharedPointer<DkResizeManipulator>::create(action);
 
     // threshold
-    action = new QAction(DkImage::loadIcon(":/nomacs/img/threshold.svg", size), QObject::tr("&Threshold..."), parent);
+    action = new QAction(DkImage::loadIcon(":/nomacs/img/threshold.svg"), QObject::tr("&Threshold..."), parent);
     action->setStatusTip(QObject::tr("Threshold the image"));
     mpls[m_threshold] = QSharedPointer<DkThresholdManipulator>::create(action);
 
     // hue/saturation
-    action = new QAction(DkImage::loadIcon(":/nomacs/img/sliders.svg", size), QObject::tr("&Hue/Saturation..."), parent);
+    action = new QAction(DkImage::loadIcon(":/nomacs/img/sliders.svg"), QObject::tr("&Hue/Saturation..."), parent);
     action->setStatusTip(QObject::tr("Change Hue and Saturation"));
     mpls[m_hue] = QSharedPointer<DkHueManipulator>::create(action);
 
     // exposure
-    action = new QAction(DkImage::loadIcon(":/nomacs/img/exposure.svg", size), QObject::tr("&Exposure..."), parent);
+    action = new QAction(DkImage::loadIcon(":/nomacs/img/exposure.svg"), QObject::tr("&Exposure..."), parent);
     action->setStatusTip(QObject::tr("Change the Exposure and Gamma"));
     mpls[m_exposure] = QSharedPointer<DkExposureManipulator>::create(action);
 
@@ -221,7 +215,7 @@ void DkManipulatorManager::loadSettings(QSettings &settings)
 {
     settings.beginGroup("Manipulators");
 
-    DkManipulatorManager::createManipulators(0);
+    DkManipulatorManager::createManipulators(nullptr);
 
     for (auto mpl : mManipulators)
         mpl->loadSettings(settings);
