@@ -26,16 +26,9 @@
  *******************************************************************************************************/
 
 #include "DkConnection.h"
-#include "DkSettings.h"
 
-#pragma warning(push, 0) // no warnings from includes - begin
-#include <QBuffer>
 #include <QByteArray>
-#include <QDebug>
-#include <QHostInfo>
-#include <QThread>
 #include <QTimer>
-#pragma warning(pop) // no warnings from includes - end
 
 namespace nmc
 {
@@ -75,7 +68,8 @@ void DkConnection::sendStartSynchronizeMessage()
         qDebug() << "mSynchronizedPeersServerPorts: " << mSynchronizedPeersServerPorts[i];
         ds << mSynchronizedPeersServerPorts[i];
     }
-    // QByteArray data = "SYNCHRONIZE" + SeparatorToken + QByteArray::number(synchronize.size()) + SeparatorToken + synchronize;
+    // QByteArray data = "SYNCHRONIZE" + SeparatorToken + QByteArray::number(synchronize.size()) + SeparatorToken +
+    // synchronize;
     QByteArray data = "STARTSYNCHRONIZE";
     data.append(SeparatorToken).append(QByteArray::number(ba.size())).append(SeparatorToken).append(ba);
     qDebug() << "sending startsynchronize:" << data;
@@ -88,9 +82,13 @@ void DkConnection::sendStopSynchronizeMessage()
     if (mState == Synchronized) { // only send message if connection is synchronized
         // qDebug() << "sending disable synchronize Message to " << this->peerName() << ":" << this->peerPort();
         QByteArray synchronize = "disable synchronizing";
-        // QByteArray data = "DISABLESYNCHRONIZE" + SeparatorToken + QByteArray::number(synchronize.size()) + SeparatorToken + synchronize;
+        // QByteArray data = "DISABLESYNCHRONIZE" + SeparatorToken + QByteArray::number(synchronize.size()) +
+        // SeparatorToken + synchronize;
         QByteArray data = "STOPSYNCHRONIZE";
-        data.append(SeparatorToken).append(QByteArray::number(synchronize.size())).append(SeparatorToken).append(synchronize);
+        data.append(SeparatorToken)
+            .append(QByteArray::number(synchronize.size()))
+            .append(SeparatorToken)
+            .append(synchronize);
         if (write(data) == data.size())
             mIsSynchronizeMessageSent = false;
         mState = ReadyForUse;
@@ -100,10 +98,12 @@ void DkConnection::sendStopSynchronizeMessage()
 void DkConnection::sendNewTitleMessage(const QString &newtitle)
 {
     mCurrentTitle = newtitle;
-    // qDebug() << "sending new Title (\"" << newtitle << "\") Message to " << this->peerName() << ":" << this->peerPort();
+    // qDebug() << "sending new Title (\"" << newtitle << "\") Message to " << this->peerName() << ":" <<
+    // this->peerPort();
 
     QByteArray newTitleBA = newtitle.toUtf8();
-    // QByteArray data = "NEWTITLE" + SeparatorToken + QByteArray::number(newTitleBA.size()) + SeparatorToken + newTitleBA;
+    // QByteArray data = "NEWTITLE" + SeparatorToken + QByteArray::number(newTitleBA.size()) + SeparatorToken +
+    // newTitleBA;
     QByteArray data = "NEWTITLE";
     data.append(SeparatorToken).append(QByteArray::number(newTitleBA.size())).append(SeparatorToken).append(newTitleBA);
     write(data);
@@ -183,22 +183,22 @@ bool DkConnection::readProtocolHeader()
         mCurrentDataType = Greeting;
     } else if (mBuffer == synchronizeBA) {
         // qDebug() << "Synchronize received from:" << this->peerAddress() << ":" << this->peerPort();
-        mCurrentDataType = startSynchronize;
+        mCurrentDataType = StartSynchronize;
     } else if (mBuffer == disableSynchronizeBA) {
         // qDebug() << "StopSynchronize received from:" << this->peerAddress() << ":" << this->peerPort();
-        mCurrentDataType = stopSynchronize;
+        mCurrentDataType = StopSynchronize;
     } else if (mBuffer == newtitleBA) {
         // qDebug() << "New Title received from:" << this->peerAddress() << ":" << this->peerPort();
-        mCurrentDataType = newTitle;
+        mCurrentDataType = NewTitle;
     } else if (mBuffer == newtransformBA) {
         // qDebug() << "New Transform received from:" << this->peerAddress() << ":" << this->peerPort();
-        mCurrentDataType = newTransform;
+        mCurrentDataType = NewTransform;
     } else if (mBuffer == newpositionBA) {
         // qDebug() << "New Position received from:" << this->peerAddress() << ":" << this->peerPort();
-        mCurrentDataType = newPosition;
+        mCurrentDataType = NewPosition;
     } else if (mBuffer == newFileBA) {
         // qDebug() << "New File received from:" << this->peerAddress() << ":" << this->peerPort();
-        mCurrentDataType = newFile;
+        mCurrentDataType = NewFile;
     } else if (mBuffer == goodbyeBA) {
         // qDebug() << "Goodbye received from:" << this->peerAddress() << ":" << this->peerPort();
         mCurrentDataType = GoodBye;
@@ -302,7 +302,8 @@ void DkConnection::checkState()
             sendGreetingMessage(mCurrentTitle);
 
         mState = ReadyForUse;
-        mPortOfPeer = peerPort(); // save peer port ... otherwise connections where this instance is server can not be removed from peerList
+        mPortOfPeer = peerPort(); // save peer port ... otherwise connections where this instance is server can not be
+                                  // removed from peerList
 
         readGreetingMessage();
 
@@ -312,7 +313,7 @@ void DkConnection::checkState()
         return;
     }
 
-    if (mState == ReadyForUse && mCurrentDataType == startSynchronize) {
+    if (mState == ReadyForUse && mCurrentDataType == StartSynchronize) {
         if (!hasEnoughData())
             return;
 
@@ -352,7 +353,7 @@ void DkConnection::checkState()
         return;
     }
 
-    if (mState == Synchronized && mCurrentDataType == stopSynchronize) {
+    if (mState == Synchronized && mCurrentDataType == StopSynchronize) {
         mState = ReadyForUse;
         this->mIsSynchronizeMessageSent = false;
         emit connectionStopSynchronize(this);
@@ -417,10 +418,10 @@ bool DkConnection::readDataTypeIntoBuffer()
 void DkConnection::processData()
 {
     switch (mCurrentDataType) {
-    case newTitle:
+    case NewTitle:
         emit connectionTitleHasChanged(this, QString::fromUtf8(mBuffer));
         break;
-    case newPosition: {
+    case NewPosition: {
         if (mState == Synchronized) {
             QRect rect;
             bool opacity;
@@ -433,7 +434,7 @@ void DkConnection::processData()
         }
         break;
     }
-    case newTransform: {
+    case NewTransform: {
         if (mState == Synchronized) {
             QTransform transform;
             QTransform imgTransform;
@@ -446,7 +447,7 @@ void DkConnection::processData()
         }
         break;
     }
-    case newFile: {
+    case NewFile: {
         if (mState == Synchronized) {
             qint16 op;
             QString filename;
