@@ -31,7 +31,6 @@
 #include "DkMessageBox.h"
 #include "DkMetaData.h"
 #include "DkMetaDataWidgets.h"
-#include "DkPluginInterface.h"
 #include "DkPluginManager.h"
 #include "DkSettings.h"
 #include "DkThumbsWidgets.h"
@@ -39,19 +38,15 @@
 #include "DkViewPort.h"
 #include "DkWidgets.h"
 
-#pragma warning(push, 0) // no warnings from includes - begin
 #include <QGridLayout>
 #include <QMouseEvent>
 #include <QSharedPointer>
 #include <QStackedLayout>
-#include <QTransform>
-#include <QtGlobal>
-#pragma warning(pop) // no warnings from includes - end
 
 namespace nmc
 {
 // DkControlWidget --------------------------------------------------------------------
-DkControlWidget::DkControlWidget(DkViewPort *parent, Qt::WindowFlags flags)
+DkControlWidget::DkControlWidget(DkThumbLoader *thumbLoader, DkViewPort *parent, Qt::WindowFlags flags)
     : DkWidget(parent, flags)
 {
     mViewport = parent;
@@ -62,7 +57,7 @@ DkControlWidget::DkControlWidget(DkViewPort *parent, Qt::WindowFlags flags)
     mCropWidget = new DkCropWidget(QRectF(), this);
 
     // thumbnails, metadata
-    mFilePreview = new DkFilePreview(this, flags);
+    mFilePreview = new DkFilePreview(thumbLoader, this, flags);
     mMetaDataInfo = new DkMetaDataHUD(this);
     mZoomWidget = new DkZoomWidget(this);
     mPlayer = new DkPlayer(this);
@@ -70,17 +65,18 @@ DkControlWidget::DkControlWidget(DkViewPort *parent, Qt::WindowFlags flags)
 
     mFolderScroll = new DkFolderScrollBar(this);
 
-    // file info - overview
+    // brief file info + ratingR
     mFileInfoLabel = new DkFileInfoLabel(this);
-    mRatingLabel = new DkRatingLabelBg(2, this, flags);
+
+    // notes
     mCommentWidget = new DkCommentWidget(this);
 
     // delayed info
     mDelayedInfo = new DkDelayedMessage(this); // TODO: make a nice constructor
 
     // info labels
-    mBottomLabel = new DkLabelBg(this, "");
-    mBottomLeftLabel = new DkLabelBg(this, "");
+    mBottomLabel = new DkLabelBg("", this);
+    mBottomLeftLabel = new DkLabelBg("", this);
 
     // wheel label
     QPixmap wp = QPixmap(":/nomacs/img/thumbs-move.svg");
@@ -124,7 +120,6 @@ void DkControlWidget::init()
     // some adjustments
     mBottomLabel->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
     mBottomLeftLabel->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
-    mRatingLabel->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
     mZoomWidget->setContentsMargins(10, 10, 0, 0);
     mCropWidget->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
     mCommentWidget->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
@@ -141,19 +136,19 @@ void DkControlWidget::init()
     mFolderScroll->registerAction(am.action(DkActionManager::menu_panel_scroller));
 
     // dummy - needed for three equal columns @markus: do not delete!
-    QWidget *dw = new QWidget(this);
+    auto *dw = new QWidget(this);
     dw->setMouseTracking(true);
-    QBoxLayout *dLayout = new QBoxLayout(QBoxLayout::LeftToRight, dw);
+    auto *dLayout = new QBoxLayout(QBoxLayout::LeftToRight, dw);
     dLayout->setContentsMargins(0, 0, 0, 0);
     dLayout->addWidget(mBottomLabel);
     dLayout->addStretch();
 
     // zoom widget
-    QWidget *bw = new QWidget(this);
+    auto *bw = new QWidget(this);
     bw->setMouseTracking(true);
     // bw->setMinimumHeight(40);
     // bw->setMaximumHeight(80);
-    QVBoxLayout *zLayout = new QVBoxLayout(bw);
+    auto *zLayout = new QVBoxLayout(bw);
     zLayout->setAlignment(Qt::AlignBottom);
     zLayout->setContentsMargins(0, 0, 0, 20);
     zLayout->setSpacing(0);
@@ -162,9 +157,9 @@ void DkControlWidget::init()
     zLayout->addWidget(mCommentWidget);
 
     // left column widget
-    QWidget *leftWidget = new QWidget(this);
+    auto *leftWidget = new QWidget(this);
     leftWidget->setMouseTracking(true);
-    QBoxLayout *ulLayout = new QBoxLayout(QBoxLayout::TopToBottom, leftWidget);
+    auto *ulLayout = new QBoxLayout(QBoxLayout::TopToBottom, leftWidget);
     ulLayout->setContentsMargins(0, 0, 0, 0);
     ulLayout->setSpacing(0);
     ulLayout->addWidget(mZoomWidget);
@@ -173,45 +168,44 @@ void DkControlWidget::init()
     ulLayout->addWidget(dw);
 
     // center column
-    QWidget *center = new QWidget(this);
+    auto *center = new QWidget(this);
     center->setMouseTracking(true);
-    QVBoxLayout *cLayout = new QVBoxLayout(center);
+    auto *cLayout = new QVBoxLayout(center);
     cLayout->setContentsMargins(0, 0, 0, 0);
     cLayout->setAlignment(Qt::AlignBottom);
     cLayout->addWidget(mPlayer);
 
     // rating widget
-    QWidget *rw = new QWidget(this);
+    auto *rw = new QWidget(this);
     rw->setMouseTracking(true);
     rw->setMinimumSize(0, 0);
-    QBoxLayout *rLayout = new QBoxLayout(QBoxLayout::RightToLeft, rw);
+    auto *rLayout = new QBoxLayout(QBoxLayout::RightToLeft, rw);
     rLayout->setContentsMargins(0, 0, 0, 17);
-    rLayout->addWidget(mRatingLabel);
     rLayout->addStretch();
 
     // file info
-    QWidget *fw = new QWidget(this);
+    auto *fw = new QWidget(this);
     fw->setContentsMargins(0, 0, 0, 30);
     fw->setMouseTracking(true);
     fw->setMinimumSize(0, 0);
-    QBoxLayout *rwLayout = new QBoxLayout(QBoxLayout::RightToLeft, fw);
+    auto *rwLayout = new QBoxLayout(QBoxLayout::RightToLeft, fw);
     rwLayout->setContentsMargins(0, 0, 0, 0);
     rwLayout->addWidget(mFileInfoLabel);
     rwLayout->addStretch();
 
     // right column
-    QWidget *hw = new QWidget(this);
+    auto *hw = new QWidget(this);
     hw->setContentsMargins(0, 10, 10, 0);
     hw->setMouseTracking(true);
-    QBoxLayout *hwLayout = new QBoxLayout(QBoxLayout::RightToLeft, hw);
+    auto *hwLayout = new QBoxLayout(QBoxLayout::RightToLeft, hw);
     hwLayout->setContentsMargins(0, 0, 0, 0);
     hwLayout->addWidget(mHistogram);
     hwLayout->addStretch();
 
     // right column
-    QWidget *rightWidget = new QWidget(this);
+    auto *rightWidget = new QWidget(this);
     rightWidget->setMouseTracking(true);
-    QBoxLayout *lrLayout = new QBoxLayout(QBoxLayout::TopToBottom, rightWidget);
+    auto *lrLayout = new QBoxLayout(QBoxLayout::TopToBottom, rightWidget);
     lrLayout->setContentsMargins(0, 0, 0, 0);
     lrLayout->addWidget(hw);
     lrLayout->addStretch();
@@ -239,7 +233,8 @@ void DkControlWidget::init()
     mHudLayout->addWidget(rightWidget, ver_center, right, 1, 1);
     mHudLayout->addWidget(mFolderScroll, top_scroll, left_thumbs, 1, hor_pos_end);
 
-    //// we need to put everything into extra widgets (which are exclusive) in order to handle the mouse events correctly
+    //// we need to put everything into extra widgets (which are exclusive) in order to handle the mouse events
+    /// correctly
     // QHBoxLayout* editLayout = new QHBoxLayout(widgets[crop_widget]);
     // editLayout->setContentsMargins(0,0,0,0);
     // editLayout->addWidget(cropWidget);
@@ -264,19 +259,16 @@ void DkControlWidget::connectWidgets()
         return;
 
     // thumbs widget
-    connect(mFilePreview, &DkFilePreview::loadFileSignal, mViewport, QOverload<const QString &>::of(&DkViewPort::loadFile));
+    connect(mFilePreview,
+            &DkFilePreview::loadFileSignal,
+            mViewport,
+            QOverload<const QString &>::of(&DkViewPort::loadFile));
     connect(mFilePreview, &DkFilePreview::changeFileSignal, mViewport, &DkViewPort::loadFileFast);
     connect(mFilePreview, &DkFilePreview::positionChangeSignal, this, &DkControlWidget::changeThumbNailPosition);
 
     // metadata widget
     connect(mMetaDataInfo, &DkMetaDataHUD::positionChangeSignal, this, &DkControlWidget::changeMetaDataPosition);
 
-    // overview
-    connect(mZoomWidget->getOverview(), &DkOverview::moveViewSignal, mViewport, &DkViewPort::moveView);
-    connect(mZoomWidget->getOverview(), &DkOverview::sendTransformSignal, mViewport, [this]() {
-        mViewport->tcpSynchronize();
-    });
-
     // zoom widget
     connect(mZoomWidget, &DkZoomWidget::zoomSignal, mViewport, &DkViewPort::zoomTo);
     connect(mViewport, &DkViewPort::zoomSignal, mZoomWidget, &DkZoomWidget::updateZoom);
@@ -287,8 +279,7 @@ void DkControlWidget::connectWidgets()
     });
 
     // rating
-    connect(mFileInfoLabel->getRatingLabel(), &DkRatingLabel::newRatingSignal, this, &DkControlWidget::updateRating);
-    connect(mRatingLabel, &DkRatingLabelBg::newRatingSignal, this, &DkControlWidget::updateRating);
+    connect(mFileInfoLabel, &DkFileInfoLabel::ratingEdited, this, &DkControlWidget::updateRating);
 
     // playing
     connect(mPlayer, &DkPlayer::previousSignal, mViewport, &DkViewPort::loadPrevFileFast);
@@ -301,11 +292,7 @@ void DkControlWidget::connectWidgets()
     });
 
     // comment widget
-    connect(mCommentWidget, &DkCommentWidget::showInfoSignal, this, [this](const QString &msg) {
-        setInfo(msg);
-    });
-    connect(mCommentWidget, QOverload<>::of(&DkCommentWidget::commentSavedSignal), this, &DkControlWidget::setCommentSaved);
-    connect(this, &DkControlWidget::imageUpdatedSignal, mCommentWidget, &DkCommentWidget::resetComment);
+    connect(mCommentWidget, &DkCommentWidget::commentSavedSignal, this, &DkControlWidget::setCommentSaved);
 
     // mViewport
     connect(mViewport, &DkViewPort::infoSignal, this, [this](const QString &msg) {
@@ -320,7 +307,10 @@ void DkControlWidget::connectWidgets()
                 QOverload<DkViewPortInterface *, bool>::of(&DkPluginActionManager::runPlugin),
                 this,
                 &DkControlWidget::setPluginWidget);
-        connect(am.pluginActionManager(), &DkPluginActionManager::applyPluginChanges, this, &DkControlWidget::applyPluginChanges);
+        connect(am.pluginActionManager(),
+                &DkPluginActionManager::applyPluginChanges,
+                this,
+                &DkControlWidget::applyPluginChanges);
     }
 
     // actions
@@ -332,12 +322,24 @@ void DkControlWidget::connectWidgets()
     connect(am.action(DkActionManager::menu_panel_exif), &QAction::toggled, this, &DkControlWidget::showMetaData);
     connect(am.action(DkActionManager::menu_panel_info), &QAction::toggled, this, &DkControlWidget::showFileInfo);
     connect(am.action(DkActionManager::menu_panel_histogram), &QAction::toggled, this, &DkControlWidget::showHistogram);
-    connect(am.action(DkActionManager::menu_panel_comment), &QAction::toggled, this, &DkControlWidget::showCommentWidget);
+    connect(am.action(DkActionManager::menu_panel_comment),
+            &QAction::toggled,
+            this,
+            &DkControlWidget::showCommentWidget);
     connect(am.action(DkActionManager::menu_panel_toggle), &QAction::toggled, this, &DkControlWidget::toggleHUD);
 }
 
-void DkControlWidget::setCommentSaved()
+void DkControlWidget::setCommentSaved(const QString &comment)
 {
+    const auto metaData = mViewport->imageContainer()->getMetaData();
+    if (!metaData || comment == metaData->getDescription()) {
+        return;
+    }
+
+    if (!metaData->setDescription(comment) && !comment.isEmpty()) {
+        setInfo(tr("Sorry, I cannot save comments for this image format."));
+        return;
+    }
     mViewport->imageContainer()->setMetaData(tr("File comment"));
 }
 
@@ -372,10 +374,13 @@ void DkControlWidget::showWidgetsSettings()
     showPreview(mFilePreview->getCurrentDisplaySetting());
     showMetaData(mMetaDataInfo->getCurrentDisplaySetting());
     showFileInfo(mFileInfoLabel->getCurrentDisplaySetting());
-    showPlayer(mPlayer->getCurrentDisplaySetting());
     showHistogram(mHistogram->getCurrentDisplaySetting());
     showCommentWidget(mCommentWidget->getCurrentDisplaySetting());
     showScroller(mFolderScroll->getCurrentDisplaySetting());
+
+    // don't show player while playing and switching modes
+    if (!mPlayer->isPlaying())
+        showPlayer(mPlayer->getCurrentDisplaySetting());
 }
 
 void DkControlWidget::toggleHUD(bool hide)
@@ -417,7 +422,8 @@ void DkControlWidget::showScroller(bool visible)
     if (visible && !mFolderScroll->isVisible())
         mFolderScroll->show();
     else if (!visible && mFolderScroll->isVisible())
-        mFolderScroll->hide(!mViewport->getImage().isNull()); // do not save settings if we have no image in the viewport
+        mFolderScroll->hide(
+            !mViewport->getImage().isNull()); // do not save settings if we have no image in the viewport
 }
 
 void DkControlWidget::showMetaData(bool visible)
@@ -429,7 +435,8 @@ void DkControlWidget::showMetaData(bool visible)
         mMetaDataInfo->show();
         qDebug() << "showing metadata...";
     } else if (!visible && mMetaDataInfo->isVisible())
-        mMetaDataInfo->hide(!mViewport->getImage().isNull()); // do not save settings if we have no image in the viewport
+        mMetaDataInfo->hide(
+            !mViewport->getImage().isNull()); // do not save settings if we have no image in the viewport
 }
 
 void DkControlWidget::showFileInfo(bool visible)
@@ -437,13 +444,11 @@ void DkControlWidget::showFileInfo(bool visible)
     if (!mFileInfoLabel)
         return;
 
-    if (visible && !mFileInfoLabel->isVisible()) {
+    if (visible && !mFileInfoLabel->isVisible())
         mFileInfoLabel->show();
-        mRatingLabel->block(mFileInfoLabel->isVisible());
-    } else if (!visible && mFileInfoLabel->isVisible()) {
-        mFileInfoLabel->hide(!mViewport->getImage().isNull()); // do not save settings if we have no image in the viewport
-        mRatingLabel->block(false);
-    }
+    else if (!visible && mFileInfoLabel->isVisible())
+        mFileInfoLabel->hide(
+            !mViewport->getImage().isNull()); // do not save settings if we have no image in the viewport
 }
 
 void DkControlWidget::showPlayer(bool visible)
@@ -513,7 +518,8 @@ void DkControlWidget::showCommentWidget(bool visible)
     if (visible && !mCommentWidget->isVisible()) {
         mCommentWidget->show();
     } else if (!visible && mCommentWidget->isVisible()) {
-        mCommentWidget->hide(!mViewport->getImage().isNull()); // do not save settings if we have no image in the mViewport
+        mCommentWidget->hide(
+            !mViewport->getImage().isNull()); // do not save settings if we have no image in the mViewport
     }
 }
 
@@ -532,12 +538,26 @@ void DkControlWidget::switchWidget(QWidget *widget)
 
     // if (mLayout->currentWidget())
     //	qDebug() << "changed to widget: " << mLayout->currentWidget();
+}
 
-    // ok, this is really nasty... however, the pImagfileInfo layout is destroyed otherwise
-    if (mLayout->currentIndex() == hud_widget && mFileInfoLabel->isVisible()) {
-        mFileInfoLabel->setVisible(false);
-        showFileInfo(true);
-    }
+void DkControlWidget::pluginClosed(bool askForSaving)
+{
+    (void)closePlugin(askForSaving);
+}
+
+void DkControlWidget::pluginMessage(const QString &msg)
+{
+    setInfo(msg);
+}
+
+void DkControlWidget::pluginLoadFile(const QString &path)
+{
+    mViewport->loadFile(path);
+}
+
+void DkControlWidget::pluginLoadImage(const QImage &img)
+{
+    mViewport->setImage(img);
 }
 
 bool DkControlWidget::closePlugin(bool askForSaving, bool force)
@@ -562,8 +582,11 @@ bool DkControlWidget::closePlugin(bool askForSaving, bool force)
             bool applyChanges = true;
 
             if (askForSaving) {
-                DkMessageBox *msgBox =
-                    new DkMessageBox(QMessageBox::Question, tr("Closing Plugin"), tr("Apply plugin changes?"), QMessageBox::Yes | QMessageBox::No, this);
+                auto *msgBox = new DkMessageBox(QMessageBox::Question,
+                                                tr("Closing Plugin"),
+                                                tr("Apply plugin changes?"),
+                                                QMessageBox::Yes | QMessageBox::No,
+                                                this);
                 msgBox->setDefaultButton(QMessageBox::Yes);
                 msgBox->setObjectName("SavePluginChanges");
 
@@ -572,7 +595,8 @@ bool DkControlWidget::closePlugin(bool askForSaving, bool force)
             }
 
             if (applyChanges)
-                pluginImage = DkImageContainerT::fromImageContainer(vPlugin->runPlugin("", mViewport->imageContainer()));
+                pluginImage = DkImageContainerT::fromImageContainer(
+                    vPlugin->runPlugin("", mViewport->imageContainer()));
         } else
             qDebug() << "[DkControlWidget] I cannot close a plugin if the image container is NULL";
     }
@@ -623,33 +647,37 @@ void DkControlWidget::setPluginWidget(DkViewPortInterface *pluginWidget, bool re
         return;
     }
 
+    // workaround some plugin bugs by disabling actions while the plugin is open
+    // TODO: this belongs in central widget as action enablement is tied to tabs
+    DkActionManager::instance().enableViewPortPluginActions(removeWidget);
+
     if (!removeWidget) {
-        mPluginViewport->setWorldMatrix(mViewport->getWorldMatrixPtr());
-        mPluginViewport->setImgMatrix(mViewport->getImageMatrixPtr());
         mPluginViewport->updateImageContainer(mViewport->imageContainer());
 
-        connect(
-            mPluginViewport,
-            &DkPluginViewPort::closePlugin,
-            this,
-            [this](bool askForSaving) {
-                closePlugin(askForSaving);
-            },
-            Qt::UniqueConnection);
-        connect(mPluginViewport, &DkPluginViewPort::loadFile, mViewport, QOverload<const QString &>::of(&DkViewPort::loadFile), Qt::UniqueConnection);
+        // NOTE: unique connections can no longer use lambdas, this was unreliable
+        // in practice and is now a fatal error in debug builds of nomacs
+        connect(mPluginViewport,
+                &DkPluginViewPort::closePlugin,
+                this,
+                &DkControlWidget::pluginClosed,
+                Qt::UniqueConnection);
+        connect(mPluginViewport,
+                &DkPluginViewPort::loadFile,
+                this,
+                &DkControlWidget::pluginLoadFile,
+                Qt::UniqueConnection);
+
+        // TODO: will this copy without using reference?
         connect(mPluginViewport,
                 &DkPluginViewPort::loadImage,
-                mViewport,
-                QOverload<QImage>::of(&DkViewPort::setImage), // TODO: will this copy without using reference?
+                this,
+                &DkControlWidget::pluginLoadImage,
+                Qt::UniqueConnection);
+        connect(mPluginViewport,
+                &DkPluginViewPort::showInfo,
+                this,
+                &DkControlWidget::pluginMessage,
                 Qt::UniqueConnection);
-        connect(
-            mPluginViewport,
-            &DkPluginViewPort::showInfo,
-            this,
-            [this](const QString &msg) {
-                setInfo(msg);
-            },
-            Qt::UniqueConnection);
     }
 
     setAttribute(Qt::WA_TransparentForMouseEvents, !removeWidget && pluginWidget->hideHUD());
@@ -662,31 +690,37 @@ void DkControlWidget::setPluginWidget(DkViewPortInterface *pluginWidget, bool re
     mViewport->setPaintWidget(dynamic_cast<QWidget *>(mPluginViewport), removeWidget);
 
     if (removeWidget) {
-        mPluginViewport = 0;
+        mPluginViewport = nullptr;
     }
 }
 
-void DkControlWidget::updateImage(QSharedPointer<DkImageContainerT> imgC)
+void DkControlWidget::updateImage(QSharedPointer<DkImageContainerT> imgC, bool updateMetadataIfNull)
 {
+    if (mImgC) {
+        disconnect(mImgC.get(),
+                   &DkImageContainerT::imageUpdatedSignal,
+                   this,
+                   &DkControlWidget::onImageContainerInternalUpdated);
+    }
+
     mImgC = imgC;
 
     if (mPluginViewport)
         mPluginViewport->updateImageContainer(imgC);
 
-    mMetaDataInfo->updateMetaData(imgC);
-
-    if (!imgC)
+    if (!imgC) {
+        if (updateMetadataIfNull) {
+            // TODO: is it more correct that we also update other related panels
+            mMetaDataInfo->setMetaData(nullptr);
+        }
         return;
+    }
 
-    QSharedPointer<DkMetaDataT> metaData = imgC->getMetaData();
-
-    QString dateString = metaData->getExifValue("DateTimeOriginal");
-    mFileInfoLabel->updateInfo(imgC->filePath(), "", dateString, metaData->getRating());
-    mFileInfoLabel->setEdited(imgC->isEdited());
-    mCommentWidget->setMetaData(metaData); // reset
-    updateRating(metaData->getRating());
-
-    connect(imgC.get(), &DkImageContainerT::imageUpdatedSignal, this, &DkControlWidget::imageUpdatedSignal);
+    onImageContainerInternalUpdated();
+    connect(imgC.get(),
+            &DkImageContainerT::imageUpdatedSignal,
+            this,
+            &DkControlWidget::onImageContainerInternalUpdated);
 }
 
 void DkControlWidget::setInfo(const QString &msg, int time, int location)
@@ -736,29 +770,32 @@ void DkControlWidget::settingsChanged()
 
 void DkControlWidget::updateRating(int rating)
 {
-    if (!mImgC)
+    if (!mImgC) {
         return;
+    }
 
-    mRatingLabel->setRating(rating);
-
-    if (mFileInfoLabel)
-        mFileInfoLabel->updateRating(rating);
-
-    QSharedPointer<DkMetaDataT> metaDataInfo = mImgC->getMetaData();
-    metaDataInfo->setRating(rating);
+    mImgC->setRating(rating);
 }
 
-void DkControlWidget::imageLoaded(bool)
+void DkControlWidget::imagePresenceChanged(bool imagePresent)
 {
+    (void)imagePresent;
+
+    // disable animations while building initial view or image is lost
+    DkFadeHelper::enableAnimations(false);
+
     showWidgetsSettings();
+
+    DkFadeHelper::enableAnimations(true);
 }
 
 void DkControlWidget::setFullScreen(bool fullscreen)
 {
     showWidgetsSettings();
 
-    if (DkSettingsManager::param().slideShow().showPlayer && fullscreen && !mPlayer->isVisible())
-        mPlayer->show(3000);
+    if (DkSettingsManager::param().slideShow().showPlayer && fullscreen && !mPlayer->getCurrentDisplaySetting()
+        && !mPlayer->isPlaying())
+        mPlayer->showTemporarily();
 }
 
 DkOverview *DkControlWidget::getOverview() const
@@ -864,4 +901,23 @@ void DkControlWidget::keyReleaseEvent(QKeyEvent *event)
     QWidget::keyReleaseEvent(event);
 }
 
+void DkControlWidget::onImageContainerInternalUpdated()
+{
+    // This is called on a signal from a valid DkImageContainerT,
+    // meaning mImgC should always be not null.
+    Q_ASSERT(mImgC);
+
+    const auto metaData = mImgC->getMetaData();
+    mMetaDataInfo->setMetaData(metaData);
+
+    if (!metaData) {
+        return;
+    }
+
+    mCommentWidget->setText(metaData->getDescription());
+
+    QString dateString = metaData->getExifValue("DateTimeOriginal");
+    mFileInfoLabel->updateInfo(mImgC->filePath(), dateString, metaData->getRating(), mImgC->isEdited());
+    mCommentWidget->setText(metaData->getDescription()); // reset
+}
 }
