@@ -36,46 +36,27 @@
 #endif
 #endif
 
-#ifdef QT_NO_DEBUG_OUTPUT
-#pragma warning(disable : 4127) // no 'conditional expression is constant' if qDebug() messages are removed
-#endif
-
-#pragma warning(push, 0) // no warnings from includes - begin
 #include <QApplication>
 #include <QCommandLineParser>
-#include <QDebug>
-#include <QDesktopServices>
-#include <QDir>
-#include <QFileInfo>
+#include <QImageReader>
 #include <QMessageBox>
 #include <QObject>
-#include <QProcess>
 #include <QTextStream>
+#include <QThread>
 #include <QTranslator>
 
-#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
-#include <QImageReader>
-#endif
-
-#pragma warning(pop) // no warnings from includes - end
-
 #include "DkCentralWidget.h"
 #include "DkNoMacs.h"
 #include "DkPluginManager.h"
 #include "DkPong.h"
 #include "DkProcess.h"
 #include "DkSettings.h"
+#include "DkShortcuts.h"
+#include "DkThemeManager.h"
 #include "DkTimer.h"
 #include "DkUtils.h"
-#include "DkViewPort.h"
-
-#include "DkDependencyResolver.h"
-#include "DkMetaData.h"
-
 #include "DkVersion.h"
-
-#include <cassert>
-#include <iostream>
+#include "DkViewPort.h"
 
 #ifdef _MSC_BUILD
 #include <shlobj.h>
@@ -93,8 +74,9 @@ int main(int argc, char *argv[])
     QCoreApplication::setOrganizationDomain("https://nomacs.org");
     QCoreApplication::setApplicationName("Image Lounge");
     QCoreApplication::setApplicationVersion(NOMACS_VERSION_STR);
-
-    QApplication::setAttribute(Qt::AA_DisableHighDpiScaling, true);
+#ifdef Q_OS_MAC
+    QApplication::setAttribute(Qt::AA_DontShowIconsInMenus, true);
+#endif
 
     QApplication app(argc, (char **)argv);
 
@@ -102,13 +84,12 @@ int main(int argc, char *argv[])
     app.setDesktopFileName("org.nomacs.ImageLounge");
 #endif
 
-#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
     QImageReader::setAllocationLimit(2048);
-#endif
 
     // init settings
     nmc::DkSettingsManager::instance().init();
-    nmc::DkMetaDataHelper::initialize(); // this line makes the XmpParser thread-save - so don't delete it even if you seem to know what you do
+    nmc::DkMetaDataHelper::initialize(); // this line makes the XmpParser thread-save - so don't delete it even if you
+                                         // seem to know what you do
 
     // uncomment this for the single instance feature...
     //// check for single instance
@@ -124,12 +105,10 @@ int main(int argc, char *argv[])
 
     parser.addHelpOption();
     parser.addVersionOption();
-    parser.addPositionalArgument("image", QObject::tr("An input image."));
+    parser.addPositionalArgument("[File|Directory...]", QObject::tr("List of files and/or directories to open"));
 
     // fullscreen (-f)
-    QCommandLineOption fullScreenOpt(QStringList() << "f"
-                                                   << "fullscreen",
-                                     QObject::tr("Start in fullscreen."));
+    QCommandLineOption fullScreenOpt(QStringList() << "f" << "fullscreen", QObject::tr("Start in fullscreen."));
     parser.addOption(fullScreenOpt);
 
     QCommandLineOption slideshowOpt(QStringList() << "slideshow", QObject::tr("Start slideshow playback"));
@@ -138,33 +117,22 @@ int main(int argc, char *argv[])
     QCommandLineOption pongOpt(QStringList() << "pong", QObject::tr("Start Pong."));
     parser.addOption(pongOpt);
 
-    QCommandLineOption privateOpt(QStringList() << "p"
-                                                << "private",
-                                  QObject::tr("Start in private mode."));
+    QCommandLineOption privateOpt(QStringList() << "p" << "private", QObject::tr("Start in private mode."));
     parser.addOption(privateOpt);
 
-    QCommandLineOption modeOpt(QStringList() << "m"
-                                             << "mode",
+    QCommandLineOption modeOpt(QStringList() << "m" << "mode",
                                QObject::tr("Set the viewing mode <mode>."),
                                QObject::tr("default | frameless | pseudocolor"));
     parser.addOption(modeOpt);
 
-    QCommandLineOption sourceDirOpt(QStringList() << "d"
-                                                  << "directory",
-                                    QObject::tr("Load all files of a <directory>."),
-                                    QObject::tr("directory"));
-    parser.addOption(sourceDirOpt);
-
-    QCommandLineOption tabOpt(QStringList() << "t"
-                                            << "tab",
-                              QObject::tr("Load <images> to tabs."),
-                              QObject::tr("images"));
-    parser.addOption(tabOpt);
-
-    QCommandLineOption batchOpt(QStringList() << "batch", QObject::tr("Batch processing of <batch-settings.pnm>."), QObject::tr("batch-settings-path"));
+    QCommandLineOption batchOpt(QStringList() << "batch",
+                                QObject::tr("Batch processing of <batch-settings.pnm>."),
+                                QObject::tr("batch-settings-path"));
     parser.addOption(batchOpt);
 
-    QCommandLineOption batchLogOpt(QStringList() << "batch-log", QObject::tr("Saves batch log to <log-path.txt>."), QObject::tr("log-path.txt"));
+    QCommandLineOption batchLogOpt(QStringList() << "batch-log",
+                                   QObject::tr("Saves batch log to <log-path.txt>."),
+                                   QObject::tr("log-path.txt"));
     parser.addOption(batchLogOpt);
 
     QCommandLineOption importSettingsOpt(QStringList() << "import-settings",
@@ -172,9 +140,19 @@ int main(int argc, char *argv[])
                                          QObject::tr("settings-path.ini"));
     parser.addOption(importSettingsOpt);
 
-    QCommandLineOption registerFilesOpt(QStringList() << "register-files", QObject::tr("Register file associations (Windows only)."));
+    QCommandLineOption registerFilesOpt(QStringList() << "register-files",
+                                        QObject::tr("Register file associations (Windows only)."));
     parser.addOption(registerFilesOpt);
 
+    QCommandLineOption listFormatsOpt(QStringList("list-formats"),
+                                      QObject::tr("List available image formats"),
+                                      "csv | plist | xdg");
+    listFormatsOpt.setFlags(QCommandLineOption::ShortOptionStyle);
+    parser.addOption(listFormatsOpt);
+
+    QCommandLineOption aboutOpt(QStringList("about"), QObject::tr("Print build information"));
+    parser.addOption(aboutOpt);
+
     parser.process(app);
 
     // CMD parser --------------------------------------------------------------------
@@ -207,6 +185,40 @@ int main(int argc, char *argv[])
         noUI = true;
     }
 
+    // extract file type information for builds/documentation/etc
+    if (parser.isSet(listFormatsOpt)) {
+        noUI = true;
+        const QString outputFormat = parser.value(listFormatsOpt);
+        QFile file;
+        if (parser.positionalArguments().count()) {
+            file.setFileName(parser.positionalArguments().first());
+            (void)file.open(QFile::WriteOnly | QFile::Truncate);
+            qInfo() << "[list-formats] writing metadata to" << file.fileName();
+        } else
+            (void)file.open(stdout, QFile::WriteOnly);
+
+        if (!file.isOpen()) {
+            qCritical() << "open failed:" << file.errorString();
+            return 1;
+        }
+        QTextStream out(&file);
+        nmc::DkFileFilterHandling::printFormats(out, outputFormat);
+    }
+
+    if (parser.isSet(aboutOpt)) {
+        noUI = true;
+        // attempts to load all runtime dependencies without showing the UI,
+        // this is used on macOS and cross-compiled windows version
+        // to discover what dylibs/dlls are being used
+        QTextStream out(stdout);
+        out << nmc::DkUtils::getBuildInfo();
+        out << "qt-imageformats:" << QImageReader::supportedImageFormats().join(",") << "\n";
+
+        nmc::DkPluginManager::instance().loadPlugins();
+        for (auto &plugin : nmc::DkPluginManager::instance().getPlugins())
+            out << "plugin:" << plugin->pluginPath() << "\n";
+    }
+
     if (noUI)
         return 0;
 
@@ -222,14 +234,10 @@ int main(int argc, char *argv[])
     nmc::DkSettingsManager::param().loadTranslation(translationNameQt, translatorQt);
     app.installTranslator(&translatorQt);
 
-    nmc::DkNoMacs *w = 0;
-    nmc::DkPong *pw = 0; // pong
+    nmc::DkNoMacs *w = nullptr;
 
-    // show pink icons if nomacs is in private mode
-    if (parser.isSet(privateOpt)) {
-        nmc::DkSettingsManager::param().display().iconColor = QColor(136, 0, 125);
+    if (parser.isSet(privateOpt))
         nmc::DkSettingsManager::param().app().privateMode = true;
-    }
 
     int mode = nmc::DkSettingsManager::param().app().appMode;
     qInfo() << "loaded app mode:" << mode;
@@ -261,6 +269,9 @@ int main(int argc, char *argv[])
 
     nmc::DkTimer dt;
 
+    // bring up theme before any widgets
+    nmc::DkThemeManager::instance().applyTheme();
+
     // initialize nomacs
     const int modeType = nmc::DkSettings::normalMode(mode);
     if (modeType == nmc::DkSettings::mode_frameless) {
@@ -270,15 +281,20 @@ int main(int argc, char *argv[])
         w = new nmc::DkNoMacsContrast();
         qDebug() << "this is the contrast nomacs...";
     } else if (parser.isSet(pongOpt)) {
-        pw = new nmc::DkPong();
-        int rVal = app.exec();
-        return rVal;
+        auto pong = std::make_unique<nmc::DkPong>();
+        return app.exec();
     } else {
         w = new nmc::DkNoMacsIpl();
     }
 
-    qInfo() << "init window: appMode:" << nmc::DkSettingsManager::param().app().currentAppMode << "maximized:" << w->isMaximized()
-            << "fullscreen:" << w->isFullScreen() << "geometry:" << w->geometry() << "windowState:" << w->windowState();
+    qInfo() << "init window: appMode:" << nmc::DkSettingsManager::param().app().currentAppMode
+            << "maximized:" << w->isMaximized() << "fullscreen:" << w->isFullScreen() << "geometry:" << w->geometry()
+            << "windowState:" << w->windowState();
+
+#ifdef Q_OS_WIN
+    // hide white flash when window first appears
+    w->setWindowOpacity(0.0);
+#endif
 
     if (nmc::DkSettings::modeIsFullscreen(mode))
         w->showFullScreen();
@@ -289,8 +305,9 @@ int main(int argc, char *argv[])
 
     const bool maximized = w->isMaximized(); // check if show* actually maximized us
 
-    qInfo() << "show window: appMode:" << nmc::DkSettingsManager::param().app().currentAppMode << "maximized:" << w->isMaximized()
-            << "fullscreen:" << w->isFullScreen() << "geometry:" << w->geometry() << "windowState:" << w->windowState();
+    qInfo() << "show window: appMode:" << nmc::DkSettingsManager::param().app().currentAppMode
+            << "maximized:" << w->isMaximized() << "fullscreen:" << w->isFullScreen() << "geometry:" << w->geometry()
+            << "windowState:" << w->windowState();
 
     while (!w->isActiveWindow() && dt.elapsed() < 5000) {
         qDebug() << "waiting for active window";
@@ -298,8 +315,12 @@ int main(int argc, char *argv[])
         qApp->processEvents();
     }
 
-    qInfo() << "active window: appMode:" << nmc::DkSettingsManager::param().app().currentAppMode << "maximized:" << w->isMaximized()
-            << "fullscreen:" << w->isFullScreen() << "geometry:" << w->geometry() << "windowState:" << w->windowState();
+    qInfo() << "active window: appMode:" << nmc::DkSettingsManager::param().app().currentAppMode
+            << "maximized:" << w->isMaximized() << "fullscreen:" << w->isFullScreen() << "geometry:" << w->geometry()
+            << "windowState:" << w->windowState();
+
+    qInfo() << "window font:" << w->font();
+    qInfo() << "system font:" << qApp->font();
 
     // Qt emulates showMaximized() on some platforms (X11), so it might not work.
     // If we try again with a visible window, it *could* work correctly (GNOME)
@@ -309,36 +330,32 @@ int main(int argc, char *argv[])
     if (w)
         w->onWindowLoaded();
 
+#ifdef Q_OS_WIN
+    // The UI is fully configured now, force repaint so we don't see a white background while loading files
+    // We can't animate smoothly here due to everything going on, so just make it opaque immediately
+    w->update();
+    qApp->processEvents();
+    w->setWindowOpacity(1.0);
+#endif
+
     qInfo() << "Initialization takes: " << dt;
 
     nmc::DkCentralWidget *cw = w->getTabWidget();
 
     bool loading = false;
 
-    if (!parser.positionalArguments().empty()) {
-        QString filePath = parser.positionalArguments()[0].trimmed();
+    for (auto &filePath : parser.positionalArguments()) {
+        if (filePath.isEmpty())
+            continue;
 
-        if (!filePath.isEmpty()) {
-            w->loadFile(QFileInfo(filePath).absoluteFilePath()); // update folder + be silent
-            loading = true;
-        }
-    }
+        if (loading)
+            cw->loadToTab(filePath);
+        else
+            cw->load(filePath);
 
-    // load directory preview
-    if (!parser.value(sourceDirOpt).trimmed().isEmpty()) {
-        cw->loadDirToTab(parser.value(sourceDirOpt));
         loading = true;
     }
 
-    // load to tabs
-    if (!parser.value(tabOpt).isEmpty()) {
-        QStringList tabPaths = parser.values(tabOpt);
-        loading = true;
-
-        for (const QString &filePath : tabPaths)
-            cw->addTab(filePath);
-    }
-
     // load recent files if there is nothing to display
     if (!loading && nmc::DkSettingsManager::param().app().showRecentFiles)
         w->showRecentFilesOnStartUp();
@@ -352,26 +369,30 @@ int main(int argc, char *argv[])
     if (cw->hasViewPort())
         cw->getViewPort()->setFocus(Qt::TabFocusReason);
 
-#ifdef Q_WS_MAC
+// since Qt5 only Q_OS_MACOS is defined, see https://doc.qt.io/qt-5/macos-issues.html#compile-time-flags
+#ifdef Q_OS_MACOS
     nmc::DkNomacsOSXEventFilter *osxEventFilter = new nmc::DkNomacsOSXEventFilter();
     app.installEventFilter(osxEventFilter);
     QObject::connect(osxEventFilter, &nmc::DkNomacsOSXEventFilter::loadFile, w, &nmc::DkNoMacs::loadFile);
 #endif
 
+    app.installEventFilter(nmc::DkShortcutEventFilter::instance());
+
     int rVal = -1;
     try {
         rVal = app.exec();
     } catch (const std::bad_alloc &) {
-        QMessageBox::critical(0, QObject::tr("Critical Error"), QObject::tr("Sorry, nomacs ran out of memory..."), QMessageBox::Ok);
+        QMessageBox::critical(nullptr,
+                              QObject::tr("Critical Error"),
+                              QObject::tr("Sorry, nomacs ran out of memory..."),
+                              QMessageBox::Ok);
     }
 
     // restore message handler, workaround for: https://github.com/nomacs/nomacs/issues/874
-    qInstallMessageHandler(0);
+    qInstallMessageHandler(nullptr);
 
     if (w)
         delete w; // we need delete so that settings are saved (from destructors)
-    if (pw)
-        delete pw;
 
     return rVal;
 }
